#ifndef GRAPH_H
#define GRAPH_H
#include <map>
#include <utility>
#include <boost/unordered_map.hpp>


namespace DataStructures{

	// for nodeId
	typedef unsigned int nodeId;

	//
	// A graph class

	template <typename T_Node, typename T_Edge>
	class Graph
	{
	public:
		typedef boost::unordered_map<nodeId, T_Node > Nodes;
		typedef std::pair <nodeId, T_Edge> Dest;
		typedef boost::unordered_multimap<nodeId, Dest> Edges;
		
		Graph(unsigned int V, unsigned int E) :  _V(V), _E(E) {};

		//  
		// get the reference to a node
		//   if the node is not found it throws an exception
		// 

		T_Node & getNode(nodeId id) throw (int)
		{
			 Nodes::iterator it = _n.find(id); 
			 if (it != _n.end())
				 return it->second;
			 else
				 throw;
		};

		//
		// insert a node, 
		//   if is is already there the node is not inserted
		//
		void insertNode(const nodeId id, const T_Node t) 
		{ _n.insert(Nodes::value_type(id, t)); };

		// get an edge
		//  return iterators
		typename Edges::iterator getEdges(nodeId id) { return _e.find(id); }
		typename Edges::iterator edgesEnd() { return _e.end(); }

		// 
		// insert an edge, 
		//   if is is already there the node is not inserted
		//
		void insertEdge(const nodeId src, const nodeId dst, T_Edge v)
		{ _e.insert(Edges::value_type(src, std::make_pair(dst, v) )); }

	private:
		unsigned int _V, _E; // number of nodes, number of edges
		Nodes _n;
		Edges _e;
	};

	//
	// A structure to represent a subset for union-find
	//
	struct subset
	{
		nodeId parent, rank;
	};


	// find set of an element i
	// (uses path compression technique)
	int find(struct subset subsets[], nodeId i);

	// A function that does union of two sets of x and y
	// (uses union by rank)
	void Union(struct subset subsets[], int x, int y);
	


};

#endif